#ifndef _SSD1928_I2C_H
#define _SSD1928_I2C_H

#include "GenericTypeDefs.h"
#include "SSD1928.h"

// I2C control from SSD1928.
// SSD1928 specific
// I2C CLK period=[2^9-(baud rate)*2^4]*MCLK period x 2
// The maximum Clock freq. for ov7660's SCCB is 150KHz
// Therefore if we want a driving clk rate of 100KHz,
// baud rate value = [2^9-(1/100)*(1/2)*80000]/2^4 = 7.
// The calculation above is valid only for MCLK=80MHz
#define SSD1928_I2C_BAUD 	7
// Definitions for I2C R/W Control registers REG[231h]
#define SSD1928_I2C_START		0x80
#define SSD1928_I2C_STOP		0x10
#define SSD1928_I2C_IDLE  		0x00
#define SSD1928_I2C_WRITE  		0x01
#define SSD1928_I2C_READ_W_ACK 	0x02
#define SSD1928_I2C_READ_WO_ACK 0x03

/*
*************************************************************************************
* Function: Initialization of I2C module for SSD1928
*
* PreCondition: none
*
* Arguments:
*
* Return: none
*
* Note:
*
*************************************************************************************
*/
void ssd1928_I2C_Init(void);

/*
*************************************************************************************
* Function: WAIT I2C module ready or Timeout
*
* PreCondition: none
*
* Arguments:	none
*
* Return: 	0 		if I2C ready
*			-1 		if timeout
*
* Note: The timeout counter is hardware dependent, and it subject to variations
*		on processor speed
*
*************************************************************************************
*/
BYTE ssd1928_I2C_Wait(void);

/*
*************************************************************************************
* Function: Function to send out the I2C Start, Stop conditions, and control
*			I2C command bits for Write, and Read w or wo acknowledgement
*
* PreCondition: none
*
* Arguments:	BYTE code list
*				SSD1928_I2C_START		0x80
*				I2C_STOP		0x10
*				I2C_IDLE  		0x00
*				I2C_WRITE  		0x01
*				I2C_READ_W_ACK 	0x02
*				I2C_READ_WO_ACK 0x03
*
* Return:		none
* Note:
*************************************************************************************
*/
void ssd1928_I2C_Ctl(BYTE code);

/*
*************************************************************************************
* Function: Function to write a single byte to I2C bus of SSD1928 via register
*			REG[230h] of SSD1928
*
* PreCondition: none
*
* Arguments:	BYTE byte to write
*
* Return:		none
*************************************************************************************
*/
void ssd1928_I2C_Write(BYTE byte);

/*
*************************************************************************************
* Function: Function to read a single byte from I2C bus of SSD1928 via register
*			REG[237h] of SSD1928
*
* PreCondition: A timeout value of 0xFFFFFFFF is a hard-code value subject
*				to variation on processor speed
*
* Arguments:	none
*
* Return:		The byte read from I2C
*				or 
*				-1 if timeout
*************************************************************************************
*/
BYTE ssd1928_I2C_Read(void);


#endif
